@@ -1,4 +1,4 @@
-#ident "@(#)conf_mg.h	3.1 95/08/30 Copyright (c) 1994 Gert Doering"
+#ident "@(#)conf_mg.h	3.3 96/07/09 Copyright (c) 1994 Gert Doering"
 
 /* all (dynamic) mgetty configuration is contained in this structure.
  * It is initialized and loaded in conf_mg.c and accessed from mgetty.c
@@ -21,6 +21,7 @@ extern struct conf_data_mgetty {
         fax_only,				/* no data */
 	modem_type,				/* auto/c2.0/cls2/data */
 	init_chat,				/* modem initialization */
+	force_init_chat,			/* for stubborn modems */
 
 	modem_check_time,			/* modem still alive? */
 	rings_wanted,				/* number of RINGs */
@@ -32,6 +33,7 @@ extern struct conf_data_mgetty {
 	ringback,				/* ringback enabled */
 	ringback_time,				/* ringback time */
 
+	ignore_carrier,				/* do not clear CLOCAL */
 	issue_file,				/* /etc/issue file */
 	prompt_waittime,			/* ms wait before prompting */
 	login_prompt,
