@@ -1,4 +1,4 @@
-#ident "@(#)conf_sf.h	3.1 95/08/30 Copyright (c) 1994 Gert Doering"
+#ident "@(#)conf_sf.h	3.3 96/03/11 Copyright (c) 1994 Gert Doering"
 
 /* all (dynamic) sendfax configuration is contained in this structure.
  * It is initialized and loaded in conf_sf.c and accessed from sendfax.c
@@ -24,6 +24,8 @@ extern struct conf_data_sendfax {
 	fax_poll_wanted,	/* cli only (-p) */
 	fax_page_header,
 	use_stdin,		/* cli only (-S) */
+        rename_files,		/* cli only (-r) */
+        acct_handle,		/* cli only (-A) */
 	end_of_config; } c;
 
 int sendfax_parse_args _PROTO(( int argc, char ** argv ));
