@@ -1,4 +1,4 @@
-#ident "@(#)faxlib.c	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)faxlib.c	3.6 96/05/26 Copyright (c) Gert Doering"
 
 /* faxlib.c
  *
@@ -19,7 +19,7 @@
 
 Modem_type modem_type = Mt_class2;	/* if uninitialized, assume class2 */
 
-char	fax_remote_id[1000];		/* remote FAX id +FTSI */
+char	fax_remote_id[40];		/* remote FAX id +FTSI */
 char	fax_param[1000];		/* transm. parameters +FDCS */
 fax_param_t	fax_par_d;		/* fax params detailed */
 char	fax_hangup = 0;
@@ -31,46 +31,24 @@ boolean fax_poll_req = FALSE;		/* poll requested */
 boolean	fhs_details = FALSE;		/* +FHS:xxx with detail info */
 int	fhs_lc, fhs_blc, fhs_cblc, fhs_lbc;
 
-/* get one line from the modem, only printable characters, terminated
- * by \r or \n. The termination charcter is *not* included
- */
-
-char * fax_get_line _P1( (fd), int fd )
-{
-    static char buffer[200];
-    int bufferp;
-    char c;
-    
-    bufferp = 0;
-    lprintf( L_JUNK, "got:" );
-    
-    do
-    {
-	if( fax_read_byte( fd, &c ) != 1 )
-	{
-	    lprintf( L_ERROR, "fax_get_line: cannot read byte, return" );
-	    return NULL;
-	}
-
-	lputc( L_JUNK, c );
-
-	if ( isprint( c ) &&
-	     bufferp < sizeof(buffer) )
-	{
-	    buffer[ bufferp++ ] = c;
-	}
-    }
-    while ( bufferp == 0 || ( c != 0x0a && c != 0x0d ) );
-
-    buffer[bufferp] = 0;
-
-    return buffer;
-}
 
 /* wait for a given modem response string,
  * handle all the various class 2 / class 2.0 status responses
  */
 
+/* copy fax station id, removing quote characters (dangerous for shell!) */
+static void fwf_copy_remote_id _P1( (id), char * id )
+{
+int w = 0;
+    while ( *id && w < sizeof(fax_remote_id)-1 )
+    {
+        if ( *id == '"' || *id == '\'' ) fax_remote_id[w++] = '_';
+                                    else fax_remote_id[w++] = *id;
+        id++;
+    }
+    fax_remote_id[w]=0;
+}
+
 static boolean fwf_timeout = FALSE;
 
 static RETSIGTYPE fwf_sig_alarm()      	/* SIGALRM handler */
@@ -101,11 +79,12 @@ int  ix;
 
     do
     {
-	line = fax_get_line( fd );
+	line = mdm_get_line( fd );
 
 	if ( line == NULL )
 	{
 	    alarm( 0 ); signal( SIGALRM, SIG_DFL );
+	    if ( fwf_timeout ) fax_hangup_code = FHUP_TIMEOUT;
 	    return ERROR;
 	}
 	
@@ -124,7 +103,7 @@ int  ix;
 	     strncmp( line, "+FPI:", 5 ) == 0 )
 	{
 	    lprintf( L_MESG, "fax_id: '%s'", line );
-	    strcpy( fax_remote_id, &line[ix] );
+	    fwf_copy_remote_id( &line[ix] );
 	}
 
 	else if ( strncmp( line, "+FDCS:", 6 ) == 0 ||
@@ -241,8 +220,12 @@ int  ix;
 	    
 	    lprintf( L_MESG, "ABORTING: line='%s'", line );
 	    fax_hangup = 1;
-	    fax_hangup_code = (strcmp( line, "BUSY" ) == 0) ? FHUP_BUSY:
-							      FHUP_ERROR;
+	    
+	    if ( strcmp(line, "BUSY") == 0 ) fax_hangup_code = FHUP_BUSY;
+	    else if (strcmp(line, "NO DIALTONE") == 0)
+	                                     fax_hangup_code = FHUP_NODIAL;
+	    else                             fax_hangup_code = FHUP_ERROR;
+	    
 	    alarm( 0 ); signal( SIGALRM, SIG_DFL );
 	    return ERROR;
 	}
@@ -294,42 +277,6 @@ int fax_command _P3( (send, expect, fd),
     return fax_wait_for( expect, fd );
 }
 
-/* simple send / expect sequence, for things that do not require
- * parsing of the modem responses, or where the side-effects are
- * unwanted.
- */
-
-int mdm_command _P2( (send, fd), char * send, int fd )
-{
-    char * l;
-    
-    if ( fax_send( send, fd ) == ERROR ) return ERROR;
-
-    /* wait for OK or ERROR, *without* side effects (as fax_wait_for
-     * would have)
-     */
-    signal( SIGALRM, fwf_sig_alarm ); alarm(20); fwf_timeout = FALSE;
-
-    do
-    {
-	l = fax_get_line( fd );
-	if ( l == NULL ) break;
-	lprintf( L_NOISE, "mdm_command: string '%s'", l );
-    }
-    while ( strcmp( l, "OK" ) != 0 && strcmp( l, "ERROR" ) != 0 );
-
-    alarm(0); signal( SIGALRM, SIG_DFL );
-    
-    if ( l == NULL || strcmp( l, "ERROR" ) == 0 )
-    {
-	lputs( L_MESG, " -> ERROR" );
-	return ERROR;
-    }
-    lputs( L_MESG, " -> OK" );
-	
-    return NOERROR;
-}
-
 /* Couple of routines to set this and that fax parameter, using class 2
  * or 2.0 commands, according to the setting of "modem_type"
  */
@@ -457,34 +404,6 @@ int fax_set_bor _P2( (fd, bor), int fd, int bor )
 }
 
 
-/* fax_read_byte
- * read one byte from "fd", with buffering
- * caveat: only one fd allowed (only one buffer), no way to flush buffers
- */
-
-int fax_read_byte _P2( (fd, c),
-		       int fd, char * c )
-{
-static char frb_buf[512];
-static int  frb_rp = 0;
-static int  frb_len = 0;
-
-    if ( frb_rp >= frb_len )
-    {
-	frb_len = read( fd, frb_buf, sizeof( frb_buf ) );
-	if ( frb_len <= 0 )
-	{
-	    if ( frb_len == 0 ) errno = 0;	/* undefined otherwise */
-	    lprintf( L_ERROR, "fax_read_byte: read returned %d", frb_len );
-	    return frb_len;
-	}
-	frb_rp = 0;
-    }
-
-    *c = frb_buf[ frb_rp++ ];
-    return 1;
-}
-
 /* find out the type of modem connected
  *
  * controlled by the "mclass" parameter ("auto", "cls2", "c2.0", "data")
