@@ -1,4 +1,4 @@
-#ident "@(#)faxrec.c	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)faxrec.c	3.14 96/10/02 Copyright (c) Gert Doering"
 
 /* faxrec.c - part of mgetty+sendfax
  *
@@ -84,6 +84,23 @@ extern  char * Device;
     fax_wait_for( "OK", 0 );
 #endif
 
+    /* if the "switchbd" flag is set wrongly, the fax_wait_for() command
+     * will time out -> write a warning to the log file and give up
+     */
+    if ( fax_hangup_code == FHUP_TIMEOUT )
+    {
+	lprintf( L_WARN, ">> The problem seen above might be caused by a wrong value of the" );
+	lprintf( L_WARN, ">> 'switchbd' option in 'mgetty.config' (currently set to '%d')", switchbd );
+
+	if ( switchbd > 0 && switchbd != 19200 )
+		lprintf( L_WARN, ">> try using 'switchbd 19200' or 'switchbd 0'");
+	else if ( switchbd > 0 )
+		lprintf( L_WARN, ">> try using 'switchbd 0'" );
+	else    lprintf( L_WARN, ">> try using 'switchbd 19200'" );
+
+	fax_hangup = 1;
+    }
+
     /* *now* set flow control (we could have set it earlier, but on SunOS,
      * enabling CRTSCTS while DCD is low will make the port hang)
      */
@@ -109,7 +126,7 @@ extern  char * Device;
     lprintf( L_NOISE, "fax receiver: hangup & end" );
 
     /* send mail to MAIL_TO */
-    if ( mail_to != NULL )
+    if ( mail_to != NULL && strlen(mail_to) != 0 )
         fax_notify_mail( pagenum, mail_to );
 
 #ifdef FAX_NOTIFY_PROGRAM
@@ -211,8 +228,9 @@ char	DevId[3];
     for ( j=0; fax_remote_id[j] != 0; j++ )
     {
 	char c = fax_remote_id[j];
-         if ( c == ' ' || c == '/' || c == '\\'|| c == '&' ||
-	      c == '(' || c == ')' || c == '>' || c == '<' )
+         if ( c == ' ' || c == '/' || c == '\\'|| c == '&' || c == ';' ||
+	      c == '(' || c == ')' || c == '>' || c == '<' || c == '|' ||
+	      c == '?' || c == '*' || c == '\''|| c == '"' || c == '`' )
 	 {
 	     if ( temp[i-1] != '-' ) temp[i++] = '-' ;
 	 }
@@ -252,6 +270,23 @@ char	DevId[3];
 	}
     }
 
+    /* do permission and owner changes as soon as possible -- security */
+
+    /* change file mode */
+    if ( file_mode != -1 &&
+	 chmod( temp, file_mode ) != 0 ) 
+    {
+	lprintf( L_ERROR, "fax_get_page_data: cannot change file mode" );
+    }
+
+    /* change file owner and group (jcp) */
+    if ( uid != -1 &&
+	 chown( temp, (uid_t) uid, (gid_t) gid ) != 0 )
+    {
+	lprintf( L_ERROR, "fax_get_page_data: cannot change owner, group" );
+    }
+
+
     /* store file name in fax_file_names */
 
     if ( fax_file_names != NULL )
@@ -284,7 +319,7 @@ char	DevId[3];
 
     while ( !fax_timeout )
     {
-	if ( fax_read_byte( fd, &c ) != 1 )
+	if ( mdm_read_byte( fd, &c ) != 1 )
 	{
 	    lprintf( L_ERROR, "error waiting for page start" );
 	    return ERROR;
@@ -306,7 +341,7 @@ char	DevId[3];
 	    alarm(FAX_PAGE_TIMEOUT);
 	}
 
-	if ( fax_read_byte( fd, &c ) != 1 )
+	if ( mdm_read_byte( fd, &c ) != 1 )
 	{
 	    ErrorCount++;
 	    lprintf( L_ERROR, "fax_get_page_data: cannot read from port (%d)!",
@@ -347,20 +382,6 @@ char	DevId[3];
 	return ERROR;
     }
 
-    /* change file mode */
-    if ( file_mode != -1 &&
-	 chmod( temp, file_mode ) != 0 ) 
-    {
-	lprintf( L_ERROR, "fax_get_page_data: cannot change file mode" );
-    }
-
-    /* change file owner and group (jcp) */
-    if ( uid != -1 &&
-	 chown( temp, (uid_t) uid, (gid_t) gid ) != 0 )
-    {
-	lprintf( L_ERROR, "fax_get_page_data: cannot change owner, group" );
-    }
-
     return NOERROR;
 }
 
@@ -454,6 +475,7 @@ char  * file_name, * p;
 char	buf[256];
 int	r;
 time_t	ti;
+extern  char * Device;
 
     lprintf( L_NOISE, "fax_notify_mail: sending mail to: %s", mail_to );
 
@@ -487,6 +509,7 @@ time_t	ti;
 	fprintf( pipe_fp, "Pages sent    : %s\n", faxpoll_server_file );
     }
 
+    fprintf( pipe_fp, "\nModem device: %s\n", Device );
     fprintf( pipe_fp, "\nCommunication parameters: %s\n", fax_param );
     fprintf( pipe_fp, "    Resolution : %s\n",
 		      fax_par_d.vr == 0? "normal" :"fine");
@@ -504,7 +527,7 @@ time_t	ti;
 
     ti = call_done - call_start;	/* time spent */
 
-    fprintf( pipe_fp, "Reception Time : %d:%d\n\n", (int) ti/60, (int) ti%60 );
+    fprintf( pipe_fp, "Reception Time : %02d:%02d\n\n", (int) ti/60, (int) ti%60 );
 
     if ( fax_hangup_code != 0 )
     {
@@ -543,6 +566,8 @@ void fax_notify_program _P1( (pagenum),
 int	r;
 char *	line;
 
+    if ( fax_file_names == NULL ) fax_file_names="";
+
     line = malloc( fax_fn_size + sizeof( FAX_NOTIFY_PROGRAM) + 100 );
     if ( line == NULL )
     {
@@ -618,7 +643,7 @@ void faxpoll_send_pages _P3( (fd, tio, pollfile),
 	lprintf( L_MESG, "fax poll: %s is (likely) G3 file", pollfile );
 
 	/* send page, no more pages to follow */
-	fax_send_page( faxpoll_server_file, tio, pp_eop, fd );
+	fax_send_page( faxpoll_server_file, NULL, tio, pp_eop, fd );
 
 	return;
     }
@@ -630,14 +655,14 @@ void faxpoll_send_pages _P3( (fd, tio, pollfile),
 
     file = fgetline( fp );
 
-    while ( file != NULL )
+    while ( file != NULL && !fax_hangup )
     {
 	/* copy filename (we need to know *before* sending the file
 	   whether it's the last one, and fgetline() uses a static buffer)
 	 */
 	
 	strncpy( buf, file, sizeof(buf)-1 );
-	buf[sizeof(buf)] = 0;
+	buf[sizeof(buf)-1] = 0;
 
 	file = fgetline( fp );
 
@@ -650,17 +675,21 @@ void faxpoll_send_pages _P3( (fd, tio, pollfile),
 	do
 	{
 	    if ( file == NULL )		/* last page */
-	        fax_send_page( buf, tio, pp_eop, fd );
+	        fax_send_page( buf, NULL, tio, pp_eop, fd );
 	    else			/* not the very last */
-	        fax_send_page( buf, tio, pp_mps, fd );
+	        fax_send_page( buf, NULL, tio, pp_mps, fd );
 	    tries++;
 
+	    if ( fax_hangup )
+	    {
+		lprintf( L_WARN, "fax poll failed: +FHNG:%d (%s)",
+			 fax_hangup_code, fax_strerror(fax_hangup_code));
+		break;
+	    }
 	    if ( fax_page_tx_status != 1 )
 	        lprintf( L_WARN, "fax poll: +FPS: %d", fax_page_tx_status );
 	}
 	while( fax_page_tx_status == 2 && tries < 2 );
-
-
     }
     fclose( fp );
 }
