@@ -1,4 +1,4 @@
-#ident "@(#)faxsend.c	3.1 95/08/30 Copyright (c) 1994 Gert Doering"
+#ident "@(#)faxsend.c	3.4 96/01/03 Copyright (c) 1994 Gert Doering"
 
 /* faxsend.c
  *
@@ -87,19 +87,20 @@ static void fax_send_panic_exit _P1( (fd), int fd )
  *
  * modem has to be in sync, waiting for at+fdt
  * page punctuation is transmitted according to "ppm"
+ * number of bytes transmitted is added to "*bytes_sent" (for statistics)
  */
-int fax_send_page _P4( (g3_file, tio, ppm, fd),
-		       char * g3_file, TIO * tio,
+int fax_send_page _P5( (g3_file, bytes_sent, tio, ppm, fd),
+		       char * g3_file, int * bytes_sent, TIO * tio,
 		       Post_page_messages ppm, int fd )
 {
-int g3fd;
-char ch;
-char buf[256];
-char wbuf[ sizeof(buf) * 2 ];
+    int g3fd;
+    char ch;
+    char buf[256];			/* read chunk */
+    char wbuf[ sizeof(buf) * 2 ];	/* worst case: size doubles */
 
-int w_total = 0;		/* total bytes written */
+    int w_total = 0;			/* total bytes written */
 
-int rc;				/* return code */
+    int rc;				/* return code */
 
     lprintf( L_NOISE, "fax_send_page(\"%s\") started...", g3_file );
 
@@ -137,7 +138,7 @@ int rc;				/* return code */
 	alarm( 20 );
 	do
 	{
-	    if ( fax_read_byte( fd, &ch ) != 1 )
+	    if ( mdm_read_byte( fd, &ch ) != 1 )
 	    {
 		lprintf( L_ERROR, "timeout waiting for XON" );
 		fprintf( stderr, "error waiting for XON!\n" );
@@ -264,7 +265,7 @@ int rc;				/* return code */
 		lprintf( L_NOISE, "input: got " );
 		do
 		{
-		    /* intentionally don't use fax_read_byte here */
+		    /* intentionally don't use mdm_read_byte here */
 		    if ( read( fd, &ch, 1 ) != 1 )
 		    {
 			lprintf( L_ERROR, "read failed" );
@@ -281,6 +282,9 @@ int rc;				/* return code */
 
     lprintf( L_MESG, "page complete, %d bytes sent", w_total );
 
+    if ( bytes_sent != NULL )
+            *bytes_sent += w_total;
+
     /* send end-of-page characters and post-page-message */
     rc = fax_send_ppm( fd, tio, ppm );
 
@@ -324,6 +328,8 @@ int fax_send_ppm _P3( (fd, tio, ppm),
 #ifndef FAX_SEND_IGNORE_CARRIER
 #ifdef sun
 	    /* HW handshake has to be off while carrier is low */
+	    /* to avoid overruns, drain buffers first (Nils Jonsson) */
+	    tio_drain_output( fd );
 	    tio_set_flow_control(fd, tio, (FAXSEND_FLOW) & FLOW_XON_OUT);
 #endif
 	    tio_carrier( tio, FALSE );
@@ -345,10 +351,10 @@ int fax_send_ppm _P3( (fd, tio, ppm),
 	    return ERROR;
 	}
 
-	/* FIXME: I think this should be fax_wait_for()! */
+	/* FIXME: I think this should be done with fax_wait_for()! */
 	do
 	{
-	    ppm_r = fax_get_line( fd );
+	    ppm_r = mdm_get_line( fd );
 	    if ( ppm_r == NULL ) return ERROR;
 
 	    /* hangup code. See fax_wait_for() for comments */
