@@ -1,4 +1,4 @@
-#ident "@(#)goodies.c	3.1 95/08/30 Copyright (c) 1993 Gert Doering"
+#ident "@(#)goodies.c	3.6 96/03/10 Copyright (c) 1993 Gert Doering"
 
 /*
  * goodies.c
@@ -14,11 +14,13 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
+#include <sys/stat.h>
 
 /* NeXTStep/86 has some byte order problems (Christian Starkjohann) */
-#if defined(NeXT) && defined(__LITTLE_ENDIAN__)
+#if defined(NeXT) && defined(__LITTLE_ENDIAN__) && !defined(NEXTSGTTY)
 # define pw_uid pw_short_pad1
 # define pw_gid pw_short_pad2
 # define gr_gid gr_short_pad
@@ -137,6 +139,29 @@ char * get_ps_args _P1 ((pid), int pid )
 
 #ifdef linux
 
+# ifdef DIALOUT_SHOW_USERNAMES
+    char procfn[30];
+    struct stat buf;
+    struct passwd *pwe;
+    static char u_logname[30];
+
+    sprintf (procfn, "/proc/%d", pid);
+    if (stat (procfn, &buf) < 0) {
+	lprintf( L_ERROR, "cannot stat %s", procfn );
+	return NULL;
+    }
+
+    if ((pwe = getpwuid (buf.st_uid)) == 0) {
+	lprintf( L_ERROR, "cannot getpwuid %d", buf.st_uid );
+	return NULL;
+    }
+	
+    strncpy( u_logname, pwe->pw_name, sizeof(u_logname)-1);
+    u_logname[sizeof(u_logname)-1]=0;
+
+    return u_logname;
+
+# else		/* standard behaviour: show command line */
     char procfn[30];
     int procfd;
     int i,l;
@@ -171,6 +196,8 @@ char * get_ps_args _P1 ((pid), int pid )
 	if ( psinfo[i] == 0 ) psinfo[i] = ' ';
 
     return psinfo;
+
+# endif /* show user name, not process cmd line */
 #endif /* linux */
 
 #if !defined(SVR4) && !defined(linux)
@@ -178,10 +205,63 @@ char * get_ps_args _P1 ((pid), int pid )
 #endif
 }
 
-#ifdef NeXT
-  /* provide dummy putenv() function */
-  int putenv( char * s ) { return 0; }
+#if defined(NeXT) || defined(NEED_PUTENV)
 
+/* provide strdup() for NEXTSTEP */
+char * strdup _P1( (src), char *src)
+{
+char * dest;
+
+    if (!src) return(NULL);
+    dest = (char *)malloc(strlen(src) + 1);
+    if (!dest) return(NULL);
+    strcpy(dest,src);
+    return(dest);
+}
+
+
+/* provide putenv() for NEXTSTEP:
+ * original code by Terrence W. Holm (tholm@uvicctr.UUCP),
+ * slightly modified by Karl Berry (karl@cs.umb.edu)
+ * contributed to mgetty by Georg Hoffleit (flight@mathi.uni-heidelberg.DE)
+ */
+
+#define  PSIZE  sizeof(char *)
+
+int putenv _P1( (entry), char *entry)
+{
+unsigned length, size;
+char * temp;
+char ** p;
+char ** new_environ;
+
+    temp = strchr(entry,'=');
+    if ( temp == 0 ) return( -1 );
+
+    length = (unsigned) (temp - entry + 1);
+
+    for ( p=environ; *p != 0 ; p++ )
+	if ( strncmp( entry, *p, length ) == 0 ) {
+	    *p = entry;
+	    return( 0 );
+	}
+
+    size = p - environ;
+    new_environ = (char **) malloc( (size+2)*PSIZE );
+
+    if ( new_environ == (char **) NULL )
+	return( -1 );
+
+    memcpy ((char *) new_environ, (const char *) environ, size*PSIZE );
+    new_environ[size]   = entry;
+    new_environ[size+1] = NULL;
+    environ = new_environ;
+
+    return(0);
+}
+#endif /* NEED_PUTENV */
+
+#ifdef NeXT
   /* provide function to repair broken tty settings
    * mega-ugly, but it seems to work
    * code provided by Christian Starkjohann <cs@ecs.co.at>
@@ -190,13 +270,16 @@ char * get_ps_args _P1 ((pid), int pid )
 # include <sgtty.h>
 void    NeXT_repair_line(int fd)
 {
-    struct sgttyb   sg;
     int             bitset = LPASS8 | LPASS8OUT;
     int             bitclr = LNOHANG;
     
+#ifndef NEXTSGTTY		/* needed only for broken POSIX subsystem */
+    struct sgttyb   sg;
+
     ioctl(fd, TIOCGETP, &sg);
     sg.sg_flags |= EVENP | ODDP;
     ioctl(fd, TIOCSETP, &sg);
+#endif
     ioctl(fd, TIOCLBIS, &bitset);
     ioctl(fd, TIOCLBIC, &bitclr);
 }
