@@ -1,9 +1,9 @@
-#ident "@(#)logfile.c	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)logfile.c	3.5 96/07/01 Copyright (c) Gert Doering"
 
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
-#ifndef NeXT
+#if !defined(NeXT) || defined(NEXTSGTTY)
 # include <varargs.h>
 #endif
 #include <sys/types.h>
@@ -25,9 +25,9 @@ int syslog _PROTO(( int, char *, ... ));
 
 #endif
 
-/* on NeXTstep, we have to use this *UGLY* way to cheat varargs/stdarg
+/* on NeXTstep(POSIX), we have to use this *UGLY* way to cheat varargs/stdarg
  */
-#ifdef NeXT
+#if defined(NeXT) && !defined(NEXTSGTTY)
 # define va_alist a1,a2,a3,a4,a5,a6,a7,a8,a9
 # define va_dcl long a1,a2,a3,a4,a5,a6,a7,a8,a9;
 # define vsprintf(buf,fmt,v) sprintf((buf),(fmt),a1,a2,a3,a4,a5,a6,a7,a8,a9)
@@ -177,7 +177,7 @@ int lputs _P2((level, s), int level, char * s )
 int retcode = 0;
     if ( log_fp != NULL && level <= log_level )
     {
-	retcode = fputs( s, log_fp );
+	retcode = fputs( s!=NULL? s : "(NULL)", log_fp );
 	fflush( log_fp );
     }
     return retcode;
@@ -255,7 +255,20 @@ int     errnr;
 #endif
     }
 
-    vsprintf( ws, format, pvar );
+    /* Marc's hack to get different verbosity levels on different
+     * intendation levels
+     *!!!! ugly. Rewrite some day.
+     */
+    ws[0] = ' ';
+    ws[1] = ' ';
+
+    if (level == L_NOISE)
+     vsprintf( &ws[1], format, pvar );
+    else if (level == L_JUNK)
+     vsprintf( &ws[2], format, pvar );
+    else
+     vsprintf( &ws[0], format, pvar );
+    
     va_end( pvar );
 
     ti = time(NULL); tm = localtime(&ti);
@@ -287,6 +300,8 @@ int     errnr;
 #ifdef SYSLOG
 	syslog( level == L_FATAL? LOG_ALERT: LOG_ERR, "%s: %m", ws );
 #endif
+
+#ifndef SYSLOG
 	if ( level == L_FATAL )		/* write to console */
 	{
 	    FILE * cons_fp;
@@ -303,6 +318,7 @@ int     errnr;
 		mail_logfile = TRUE;
 	    }
 	}
+#endif
     }	/* end if ( L_ERROR or L_FATAL ) */
     fflush(log_fp);
 
