@@ -1,4 +1,4 @@
-#ident "@(#)login.c	3.1 95/08/30 Copyright (C) 1993 Gert Doering"
+#ident "@(#)login.c	3.3 96/07/01 Copyright (C) 1993 Gert Doering"
 
 
 /* login.c
@@ -18,7 +18,7 @@
 #include <sys/stat.h>
 
 /* NeXTStep/86 has some byte order problems (Christian Starkjohann) */
-#if defined(NeXT) && defined(__LITTLE_ENDIAN__)
+#if defined(NeXT) && defined(__LITTLE_ENDIAN__) && !defined(NEXTSGTTY)
 # define pw_uid pw_short_pad1
 # define pw_gid pw_short_pad2
 # define gr_gid gr_short_pad
@@ -298,6 +298,13 @@ void login_dispatch _P1( (user), char * user )
 	Device, getpid(), CallerId, Connect, CallName,
 	cmd, user );
 
+    /* set a couple of environment variables (mainly useful for "special"
+     * logins, like Fido and AutoPPP, because /bin/login reconstructs its
+     * environment from scratch) - not general enough, though.
+     */
+    set_env_var( "CALLER_ID", CallerId );
+    set_env_var( "CONNECT", Connect );
+
     /* execute login */
     execv( cmd, argv );
 
