@@ -1,4 +1,4 @@
-#ident "@(#)mg_m_init.c	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)mg_m_init.c	3.8 96/09/15 Copyright (c) Gert Doering"
 
 /* mg_m_init.c - part of mgetty+sendfax
  *
@@ -22,7 +22,7 @@
 #include "policy.h"
 #include "fax_lib.h"
 
-#if (defined(M_XENIX) && !defined(M_UNIX))
+#if (defined(M_XENIX) && !defined(M_UNIX)) || defined(NEXTSGTTY)
 #define O_NOCTTY 0
 #endif
 
@@ -35,18 +35,46 @@ static int init_chat_timeout = 20;
 
 /* initialize data section */
 
-int mg_init_data _P2( (fd, chat_seq), int fd, char * chat_seq[] )
+int mg_init_data _P3( (fd, chat_seq, force_chat_seq), 
+		      int fd, char * chat_seq[], char * force_chat_seq[] )
 {
     action_t what_action;
     
     if ( do_chat( fd, chat_seq, init_chat_actions,
-		 &what_action, init_chat_timeout, TRUE ) == FAIL )
+		 &what_action, init_chat_timeout, TRUE ) == SUCCESS )
     {
-	errno = ( what_action == A_TIMOUT )? EINTR: EINVAL;
-	lprintf( L_ERROR, "init chat failed, exiting..." );
-	return FAIL;
+	return SUCCESS;
     }
-    return SUCCESS;
+
+    /* if init_chat failed because the modem didn't respond, and we have 
+     * a "force_chat" sequence, try this.
+     * (force_chat might contain DLE ETX for voice modems, or just plain 
+     * simple +++ATH0 for data modems (mis-)configured with AT&D0)
+     */
+    if ( what_action == A_TIMOUT && force_chat_seq != NULL )
+    {
+	lprintf( L_WARN, "init chat timed out, trying force-init-chat" );
+
+	if ( do_chat( fd, force_chat_seq, init_chat_actions,
+		     &what_action, init_chat_timeout, TRUE ) == SUCCESS ||
+	     what_action != A_TIMOUT )
+        {
+	    lprintf( L_NOISE, "force-init succeeded, retrying init-chat");
+
+	    clean_line(fd, 3);
+	    if ( do_chat( fd, chat_seq, init_chat_actions,
+			 &what_action, init_chat_timeout, TRUE ) == SUCCESS )
+	    {
+		return SUCCESS;
+	    }
+        }
+    }
+
+    /* either no force_chat available, or that didn't help either: BARF!
+     */
+    errno = ( what_action == A_TIMOUT )? EINTR: EINVAL;
+    lprintf( L_ERROR, "init chat failed, exiting..." );
+    return FAIL;
 }
 
 
@@ -54,7 +82,7 @@ int mg_init_data _P2( (fd, chat_seq), int fd, char * chat_seq[] )
 
 /* initialize fax section */
 
-int mg_init_fax _P4( (fd, mclass, fax_id),
+int mg_init_fax _P4( (fd, mclass, fax_id, fax_only),
 		      int fd, char * mclass, char * fax_id, boolean fax_only )
 {
     /* find out whether this beast is a fax modem... */
@@ -173,7 +201,7 @@ int mg_open_device _P2 ( (devname, blocking),
 	fd = open(devname, O_RDWR | O_NDELAY | O_NOCTTY );
 	if ( fd < 0 )
 	{
-	    lprintf( L_FATAL, "cannot open line" );
+	    lprintf( L_FATAL, "mod: cannot open line %s", devname );
 	    return ERROR;
 	}
 
@@ -191,10 +219,14 @@ int mg_open_device _P2 ( (devname, blocking),
 	{
 	    if ( errno == EAGAIN ) goto again;
 	    
-	    lprintf( L_FATAL, "cannot open line" );
+	    lprintf( L_FATAL, "mod: cannot open line %s", devname );
 	    return ERROR;
 	}
     }
+#ifdef NEXTSGTTY
+    /* get rid of controlling tty: on NeXT, there is no O_NOCTTY */
+    ioctl( fd, TIOCNOTTY, 0 );
+#endif
 
     /* make new fd == stdin if it isn't already */
 
@@ -203,7 +235,7 @@ int mg_open_device _P2 ( (devname, blocking),
 	(void) close(0);
 	if (dup(fd) != 0)
 	{
-	    lprintf( L_FATAL, "cannot open stdin" );
+	    lprintf( L_FATAL, "mod: cannot make %s stdin", devname );
 	    return ERROR;
 	}
     }
@@ -215,11 +247,11 @@ int mg_open_device _P2 ( (devname, blocking),
     
     if (dup(0) != 1)
     {
-	lprintf( L_FATAL, "cannot open stdout"); return ERROR;
+	lprintf( L_FATAL, "mod: cannot dup to stdout"); return ERROR;
     }
     if (dup(0) != 2)
     {
-	lprintf( L_FATAL, "cannot open stderr"); return ERROR;
+	lprintf( L_FATAL, "mod: cannot dup to stderr"); return ERROR;
     }
 
     if ( fd > 2 ) (void) close(fd);
@@ -299,6 +331,7 @@ int mg_get_device _P5( (devname, blocking_open,
 		        unsigned int portspeed)
 {
     boolean first_try = TRUE;
+    int rs_lines;
 
     /* most likely, HUPCL was set and so DTR is low right now. Give
      * modem some time to settle down.
@@ -313,6 +346,13 @@ try_again:
 	return ERROR;
     }
 
+    /* catch "standard question #17" (DCD drop -> fd invalid -> I/O error) */
+    rs_lines = tio_get_rs232_lines(STDIN);
+#ifdef linux
+    if ( rs_lines != -1 && ( rs_lines & TIO_F_DCD ) )
+	lprintf( L_MESG, "WARNING: DCD line still active, check modem settings" );
+#endif
+
     /* initialize device (hangup, raw, speed). May fail! */
     if ( mg_init_device( STDIN, toggle_dtr, toggle_dtr_waittime,
 			 portspeed ) == ERROR )
