@@ -1,4 +1,4 @@
-#ident "@(#)mg_utmp.h	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)mg_utmp.h	3.2 95/11/24 Copyright (c) Gert Doering"
 
 /* definitions for utmp reading / writing routines,
  * highly SysV / BSD dependent
@@ -20,6 +20,11 @@
 # include <utmp.h>
 #endif
 
+#ifdef _AIX
+struct utmp * getutent();		/* AIX 3.2.5 doesn't declare these */
+void setutent();
+#endif
+
 #define UT_INIT		INIT_PROCESS
 #define UT_LOGIN	LOGIN_PROCESS
 #define UT_USER		USER_PROCESS
