@@ -1,4 +1,4 @@
-#ident "@(#)mgetty.c	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)mgetty.c	3.22 96/10/05 Copyright (c) Gert Doering"
 
 /* mgetty.c
  *
@@ -19,9 +19,7 @@
 #include <sys/stat.h>
 #include <signal.h>
 
-#ifdef VOICE
-#include "voice.h"
-#endif
+#include "version.h"
 #include "mgetty.h"
 #include "policy.h"
 #include "tio.h"
@@ -31,8 +29,11 @@
 #include "config.h"
 #include "conf_mg.h"
 
-#ifndef VOICE
-#include "version.h"		/* for logging the mgetty release number */
+#ifdef VOICE
+#include "voice/include/voice.h"
+# ifndef DIST_RING
+#  define DIST_RING
+# endif
 #endif
 
 #ifdef DIST_RING
@@ -59,11 +60,21 @@ chat_action_t	ring_chat_actions[] = { { "CONNECT",	A_CONN },
 					{ "VCON",       A_VCON },
 #endif
 #ifdef DIST_RING
-					{ "RING 1",	A_RING1 },
+		/* ZyXEL */		{ "RING 1",	A_RING1 },
 					{ "RING 2",	A_RING2 },
 					{ "RING 3",	A_RING3 },
 					{ "RING 4",	A_RING4 },
 					{ "RING 5",	A_RING5 },
+		/* USR */		{ "RING A",	A_RING1 },
+					{ "RING B",	A_RING2 },
+					{ "RING C",	A_RING3 },
+					{ "RING D",	A_RING4 },
+					{ "RING E",	A_RING5 },
+		/* Supra */		{ "RING1",	A_RING1 },
+					{ "RING2",	A_RING2 },
+					{ "RING3",	A_RING3 },
+					{ "RING4",	A_RING4 },
+					{ "RING5",	A_RING5 },
 #endif
 					{ NULL,		A_FAIL } };
 
@@ -131,7 +142,8 @@ static void make_pid_file _P0( void )
 #endif
     
 
-enum { St_unknown,
+enum mgetty_States
+     { St_unknown,
        St_go_to_jail,			/* reset after unwanted call */
        St_waiting,			/* wait for activity on tty */
        St_check_modem,			/* check if modem is alive */
@@ -145,16 +157,84 @@ enum { St_unknown,
        St_incoming_fax			/* +FCON detected */
    } mgetty_state = St_unknown;
 
-void st_dialout _P0( void )
+/* called on SIGUSR2. Exit, if no user online, ignore otherwise */
+static RETSIGTYPE sig_new_config()
+{
+    signal( SIGUSR2, sig_new_config );
+    if ( mgetty_state != St_answer_phone &&
+	 mgetty_state != St_get_login &&
+	 mgetty_state != St_incoming_fax )
+    {
+	lprintf( L_AUDIT, "exit dev=%s, pid=%d, got signal USR2, exiting",
+	              Device, getpid() );
+	rmlocks();
+	exit(0);
+    }
+    lprintf( L_MESG, "Got SIGUSR2, modem is off-hook --> ignored" );
+}
+   
+enum mgetty_States st_sig_callback _P2( (pid, devname),
+				        int pid, char * devname )
+{
+    TIO tio;
+    
+    lprintf( L_MESG, "Got callback signal from pid=%d!", pid );
+
+    /* reopen device */
+    if ( mg_open_device( devname, FALSE ) == ERROR )
+    {
+	lprintf( L_FATAL, "stsc: can't reopen device" );
+	exit(0);
+    }
+
+    /* setup device (but do *NOT*!! set speed) */
+    if ( tio_get( STDIN, &tio ) == ERROR )
+    {
+	lprintf( L_FATAL, "stsc: can't get TIO" ); exit(0);
+    }
+    tio_mode_sane( &tio, c_bool( ignore_carrier ) );
+    tio_default_cc( &tio );
+    tio_mode_raw( &tio );
+    tio_set_flow_control( STDIN, &tio, DATA_FLOW );
+    if ( tio_set( STDIN, &tio ) == ERROR )
+    {
+	lprintf( L_FATAL, "stsc: can't set TIO" ); exit(0);
+    }
+    
+    /* make line controlling tty */
+    mg_get_ctty( STDIN, devname );
+
+    /* steal lock file from callback process */
+    lprintf( L_MESG, "stealing lock file from pid=%d", pid );
+    if ( steal_lock( Device, pid ) == ERROR ) return St_dialout;
+
+    /* signal user */
+    printf( "...ok\r\n" );
+
+    /* signal callback process (but give it some time to enter pause()! */
+    delay(500);
+    if ( kill( pid, SIGUSR1 ) < 0 )
+    {
+	lprintf( L_ERROR, "can't signal callback process" );
+    }
+
+    /* now give user a login prompt! */
+    return St_get_login;
+}
+
+/* line locked, parallel dialout in process.
+ *
+ * Two things can happen now:
+ *   - lock file disappears --> dialout terminated, exit(), restart
+ *   - get signal SIGUSR1 --> dialout was callback, mgetty takes over
+ */
+enum mgetty_States st_dialout _P1( (devname), char * devname )
 {
     int pid;
     
     /* the line is locked, a parallel dialout is in process */
 
-    /* close all file descriptors -> other processes can read port */
-    close(0);
-    close(1);
-    close(2);
+    virtual_ring = FALSE;			/* used to signal callback */
 
     /* write a note to utmp/wtmp about dialout, including process args
      * (don't do this on two-user-license systems!)
@@ -164,6 +244,11 @@ void st_dialout _P0( void )
     make_utmp_wtmp( Device, UT_USER, "dialout", get_ps_args(pid) );
 #endif
 
+    /* close all file descriptors -> other processes can read port */
+    close(0);
+    close(1);
+    close(2);
+
     /* this is kind of tricky: sometimes uucico dial-outs do still
        collide with mgetty. So, when my uucico times out, I do
        *immediately* restart it. The double check makes sure that
@@ -171,7 +256,16 @@ void st_dialout _P0( void )
     
     do {
 	/* wait for lock to disappear */
-	while ( checklock(Device) != NO_LOCK ) sleep(10);
+	while ( ( pid = checklock(Device) ) != NO_LOCK ) 
+	{
+	    sleep(10);
+
+	    /* virtual ring? this would mean an active callback! */
+	    if ( virtual_ring )
+	    {
+		return st_sig_callback( pid, devname );
+	    }
+	}
 	
 	/* wait a moment, then check for reappearing locks */
 	sleep(5);
@@ -206,25 +300,41 @@ int main _P2((argc, argv), int argc, char ** argv)
 
 #ifdef VOICE
     boolean	use_voice_mode = TRUE;
-    
-    voice_config( argc, argv );
 #endif
 	
-    /* startup
+    /* startup: initialize all signal handlers *NOW*
      */
     (void) signal(SIGHUP, SIG_IGN);
-    (void) signal(SIGINT, SIG_IGN);
-    (void) signal(SIGQUIT, SIG_DFL);
-    (void) signal(SIGTERM, SIG_DFL);
 
+    /* set to remove lockfile(s) and print "got signal..." message
+     */
+    (void) signal(SIGINT, sig_goodbye);
+    (void) signal(SIGQUIT, sig_goodbye);
+    (void) signal(SIGTERM, sig_goodbye);
+
+    /* sometimes it may be desired to have mgetty pick up the phone even
+       if it didn't RING often enough (because you accidently picked it up
+       manually...) or if it didn't RING at all (because you have a fax
+       machine directly attached to the modem...), so send mgetty a signal
+       SIGUSR1 and it will behave as if a RING was seen
+       In addition, this is used by the "callback" module.
+       */
+    signal( SIGUSR1, sig_pick_phone );
+
+    /* for reloading the configuration file, we need a way to tell mgetty
+       "restart, but only if no user is online". Use SIGUSR2 for that
+       */
+    signal( SIGUSR2, sig_new_config );
+
+#ifdef HAVE_SIGINTERRUPT
     /* some systems, notable BSD 4.3, have to be told that system
      * calls are not to be automatically restarted after those signals.
      */
-#ifdef HAVE_SIGINTERRUPT
     siginterrupt( SIGINT,  TRUE );
     siginterrupt( SIGALRM, TRUE );
     siginterrupt( SIGHUP,  TRUE );
     siginterrupt( SIGUSR1, TRUE );
+    siginterrupt( SIGUSR2, TRUE );
 #endif
 
     Device = "unknown";
@@ -258,13 +368,20 @@ int main _P2((argc, argv), int argc, char ** argv)
     sprintf( buf, LOG_PATH, DevID );
     log_init_paths( argv[0], buf, &Device[strlen(Device)-3] );
 
-#ifndef VOICE
-    lprintf( L_NOISE, "mgetty: %s", mgetty_version );
+#ifdef VOICE
+    lprintf( L_MESG, "vgetty: %s", vgetty_version);
 #endif
+    lprintf( L_MESG, "mgetty: %s", mgetty_version);
+    lprintf( L_NOISE, "%s compiled at %s, %s", __FILE__, __DATE__, __TIME__ );
 	    
     /* read configuration file */
     mgetty_get_config( Device );
 
+#ifdef VOICE
+    voice_config("vgetty", DevID);
+    voice_register_event_handler(vgetty_handle_event);
+#endif
+
 #ifdef USE_GETTYDEFS
     if (optind < argc)
         conf_set_string( &c.gettydefs_tag, argv[optind++] );
@@ -289,7 +406,7 @@ int main _P2((argc, argv), int argc, char ** argv)
      */
     if (checklock(Device) != NO_LOCK)
     {
-	st_dialout();
+	st_dialout(NULL);
     }
 
     /* try to lock the line
@@ -298,7 +415,7 @@ int main _P2((argc, argv), int argc, char ** argv)
 
     if ( makelock(Device) == FAIL )
     {
-	st_dialout();
+	st_dialout(NULL);
     }
 
     /* the line is mine now ...  */
@@ -325,21 +442,20 @@ int main _P2((argc, argv), int argc, char ** argv)
        - loading the gettydefs file if enabled.
        - setting port speed appropriately, if not set yet.
        */
-    gettermio(c_string(gettydefs_tag), TRUE, &tio);
+    gettermio(c_string(gettydefs_tag), TRUE, (TIO *) NULL);
 
     /* open + initialize device (mg_m_init.c) */
     if ( mg_get_device( devname, c_bool(blocking),
 		        c_bool(toggle_dtr), c_int(toggle_dtr_waittime),
 		        c_int(speed) ) == ERROR )
     {
-	lprintf( L_FATAL, "cannot get terminal line, exiting" );
-	exit( 30 );
+	lprintf( L_FATAL, "cannot get terminal line dev=%s, exiting", Device);
+	exit(30);
     }
     
     /* drain input - make sure there are no leftover "NO CARRIER"s
      * or "ERROR"s lying around from some previous dial-out
      */
-
     clean_line( STDIN, 1);
 
     /* do modem initialization, normal stuff first, then fax
@@ -349,8 +465,11 @@ int main _P2((argc, argv), int argc, char ** argv)
     else
     {
 	/* initialize data part */
-	if ( mg_init_data( STDIN, c_chat(init_chat) ) == FAIL )
+	if ( mg_init_data( STDIN, c_chat(init_chat),
+	                          c_chat(force_init_chat) ) == FAIL )
 	{
+	    lprintf( L_AUDIT, "failed in mg_init_data, dev=%s, pid=%d",
+	                      Device, getpid() );
 	    rmlocks();
 	    exit(1);
 	}
@@ -374,20 +493,20 @@ int main _P2((argc, argv), int argc, char ** argv)
 		faxpoll_server_init( STDIN, c_string(fax_server_file) );
 	    }
 	}
-#ifdef VOICE
-	if ( mg_init_voice( STDIN ) == FAIL )
-	{
-	    use_voice_mode = FALSE;
-	} else {
-	    use_voice_mode = TRUE;
 
-	    /* With external modems, the auto-answer LED can be used
-	     * to show a status flag. vgetty uses this to indicate
-	     * that new messages have arrived.
-	     */
-	    voice_message_light();
-	}
-#endif
+#ifdef VOICE
+    voice_fd = STDIN;
+    voice_init();
+
+    if ( use_voice_mode ) {
+	/* With external modems, the auto-answer LED can be used
+	 * to show a status flag. vgetty uses this to indicate
+	 * that new messages have arrived.
+	 */
+	vgetty_message_light();
+    }
+#endif /* VOICE */
+
     }
 
     /* wait .3s for line to clear (some modems send a \n after "OK",
@@ -401,27 +520,12 @@ int main _P2((argc, argv), int argc, char ** argv)
 
     rmlocks();	
 
-    /* sometimes it may be desired to have mgetty pick up the phone even
-       if it didn't RING often enough (because you accidently picked it up
-       manually...) or if it didn't RING at all (because you have a fax
-       machine directly attached to the modem...), so send mgetty a signal
-       SIGUSR1 and it will behave as if a RING was seen
-       */
-    
-    signal( SIGUSR1, sig_pick_phone );
-
 #if ( defined(linux) && defined(NO_SYSVINIT) ) || defined(sysV68)
     /* on linux, "simple init" does not make a wtmp entry when you
      * log so we have to do it here (otherwise, "who" won't work) */
     make_utmp_wtmp( Device, UT_INIT, "uugetty", NULL );
 #endif
 
-    /* set to remove lockfile(s) on certain signals (SIGHUP is ignored)
-     */
-    (void) signal(SIGINT, sig_goodbye);
-    (void) signal(SIGQUIT, sig_goodbye);
-    (void) signal(SIGTERM, sig_goodbye);
-
     /* sleep... waiting for activity */
     mgetty_state = St_waiting;
 
@@ -518,10 +622,10 @@ int main _P2((argc, argv), int argc, char ** argv)
 		mgetty_state = St_go_to_jail; break;
 	    }
 
-	    lprintf( L_FATAL, "modem on %s doesn't react!", devname );
+	    lprintf( L_FATAL, "modem on dev=%s doesn't react!", Device );
 
 	    /* give up */
-	    exit( 30 );
+	    exit(30);
 
 	    break;
 		
@@ -574,7 +678,8 @@ int main _P2((argc, argv), int argc, char ** argv)
 
 
 	  case St_dialout:
-	    st_dialout();	/* does /NOT/ return */
+	    /* wait for lock file to disappear *OR* for callback in progress */
+	    mgetty_state = st_dialout(devname);
 	    break;
 
 	  case St_wait_for_RINGs:
@@ -608,7 +713,7 @@ int main _P2((argc, argv), int argc, char ** argv)
 #ifdef VOICE
 	    if ( use_voice_mode ) {
 		/* modify, if toll saver, or in vgetty answer-file */
-		voice_rings(&rings_wanted);
+		vgetty_rings(&rings_wanted);
 	    }
 #endif /* VOICE */
 
@@ -649,7 +754,10 @@ Ring_got_action:
 		    rmlocks();		/* line is free again */
 		    exit(0);		/* let init restart mgetty */
 		}
-		lprintf( L_AUDIT, "phone stopped ringing (rings=%d)", rings );
+		if ( c_bool(ringback) )
+		    lprintf( L_AUDIT, "missed ringback!" );
+		else
+		    lprintf( L_AUDIT, "phone stopped ringing (rings=%d)", rings );
 		mgetty_state = St_go_to_jail;
 		break;
 	      case A_CONN:		/* CONNECT */
@@ -659,7 +767,7 @@ Ring_got_action:
 		mgetty_state = St_incoming_fax; break;
 #ifdef VOICE
 	      case A_VCON:
-		voice_button(rings);
+		vgetty_button(rings);
 		use_voice_mode = FALSE;
 		mgetty_state = St_answer_phone;
 		break;
@@ -717,7 +825,7 @@ Ring_got_action:
 		   called. If the function returns, the modem is ready
 		   to be connected in DATA mode with ATA. */
 		
-		voice_answer(rings, rings_wanted, what_action );
+		vgetty_answer(rings, rings_wanted, what_action );
 	    }
 #endif /* VOICE */
 
@@ -803,11 +911,17 @@ Ring_got_action:
     
     clean_line( STDIN, 3);
 
-    /* honor carrier now: terminate if modem hangs up prematurely
-     */
     tio_get( STDIN, &tio );
-    tio_carrier( &tio, TRUE );
-    tio_set( STDIN, &tio );
+    /* honor carrier now: terminate if modem hangs up prematurely
+     * (can be bypassed if modem / serial port broken)
+     */
+    if ( !c_bool( ignore_carrier ) )
+    {
+	tio_carrier( &tio, TRUE );
+	tio_set( STDIN, &tio );
+    }
+    else
+        lprintf( L_MESG, "warning: carrier signal is ignored" );
     
     /* make utmp and wtmp entry (otherwise login won't work)
      */
@@ -824,12 +938,16 @@ Ring_got_action:
     {
 	/* set ttystate for /etc/issue ("before" setting) */
 	gettermio(c_string(gettydefs_tag), TRUE, &tio);
-#ifdef sun
-	/* we have carrier, assert data flow control */
-	tio_set_flow_control( STDIN, &tio, DATA_FLOW );
-#endif
+
+	/* we have carrier, assert flow control (including HARD and IXANY!) */
+	tio_set_flow_control( STDIN, &tio, DATA_FLOW | FLOW_XON_IXANY );
 	tio_set( STDIN, &tio );
 	
+#ifdef NeXT
+	/* work around NeXT's weird problems with POSIX termios vs. sgtty */
+	NeXT_repair_line(STDIN);
+#endif
+	
 	fputc('\r', stdout);	/* just in case */
 	
 	if (c_isset(issue_file))
@@ -893,11 +1011,13 @@ Ring_got_action:
 	        { sprintf( t, "TERM=%s", c_string(termtype) ); putenv(t); }
 	}
 
-#ifdef NeXT
-	/* work around NeXT's weird problems with POSIX termios vs. sgtty */
-	NeXT_repair_line(STDIN);
-#endif
-	
+	/* catch "standard question #29" (DCD low -> /bin/login gets stuck) */
+	i = tio_get_rs232_lines(STDIN);
+	if ( i != -1 && (( i & TIO_F_DCD ) == 0 ) )
+	{
+	    lprintf( L_WARN, "WARNING: starting login while DCD is low!" );
+	}
+
 	/* hand off to login dispatcher (which will call /bin/login) */
 	login_dispatch( buf );
 
@@ -917,7 +1037,7 @@ gettermio _P3 ((id, first, tio), char *id, boolean first, TIO *tio )
 #endif
 
     /* default setting */
-    tio_mode_sane( tio, FALSE );
+    if ( tio != NULL ) tio_mode_sane( tio, c_bool( ignore_carrier ) );
     rp = NULL;
 
 #ifdef USE_GETTYDEFS
@@ -940,6 +1060,7 @@ gettermio _P3 ((id, first, tio), char *id, boolean first, TIO *tio )
 		 c.speed.flags == C_PRESET )	/* -> use gettydefs */
 	        conf_set_int( &c.speed, tio_get_speed( &(gdp->before)) );
 	} else		/* "after" -> set termio flags *BUT NOT* speed */
+            if ( tio != NULL )
 	{
 	    *tio = gdp->after;
 	    tio_set_speed( tio, c_int(speed) );
