@@ -1,4 +1,4 @@
-#ident "@(#)sendfax.c	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)sendfax.c	3.12 96/09/15 Copyright (c) Gert Doering"
 
 /* sendfax.c
  *
@@ -18,7 +18,11 @@
 #endif
 #include <signal.h>
 #include <sys/types.h>
+#ifndef ENOENT
+# include <errno.h>
+#endif
 
+#include "version.h"
 #include "mgetty.h"
 #include "tio.h"
 #include "policy.h"
@@ -38,18 +42,22 @@ extern time_t	call_start;			/* for accounting */
 /* seems to missing nearly everywhere */
 time_t	time _PROTO(( long * tloc ));
 
-void exit_usage _P1( (program),
-		     char * program )
+void exit_usage _P2( (program, msg ),
+		     char * program, char * msg )
 {
+    if ( msg != NULL )
+        fprintf( stderr, "%s: %s\n", program, msg );
+    
     fprintf( stderr,
 	     "usage: %s [options] <fax-number> <page(s) in g3-format>\n", program);
     fprintf( stderr,
-	     "\tvalid options: -p, -h, -v, -l <device(s)>, -x <debug>, -n, -S\n");
+	     "\tvalid options: -p, -h, -v, -l <device(s)>, -x <debug>, -n, -S, -r\n");
+    lprintf( L_AUDIT, "failed: command line error" );
     exit(1);
 }
 
 TIO fax_tio;
-char * Device;
+char *Device = "unset";
 
 int fax_open_device _P2( (fax_tty, use_stdin),
 			 char * fax_tty, boolean use_stdin )
@@ -232,8 +240,10 @@ RETSIGTYPE fax_sig_goodbye _P1( (signo), int signo )
 {
     if ( call_start == 0 ) call_start = time(NULL);
     
-    lprintf( L_AUDIT, "failed, pid=%d, time=%ds, got signal %d, exiting...", 
-	     getpid(), ( time(NULL)-call_start ), signo );
+    lprintf( L_AUDIT, 
+	     "failed: got signal %d, pid=%d, dev=%s, time=%ds, acct=\"%s\"", 
+	     signo, getpid(), Device,
+	     ( time(NULL)-call_start ), c_string(acct_handle));
     rmlocks();
     exit(15);				/* will close the fax device */
 }
@@ -247,6 +257,10 @@ int main _P2( (argc, argv),
     int	i;
     int	tries;			/* number of unsuccessful tries */
 
+    int	total_bytes = 0;	/* number of bytes sent */
+    int total_pages = 0;	/* number of pages (files) sent */
+    int total_resent= 0;	/* number of pages resent */
+
 
     /* initialize logging */
     log_init_paths( argv[0], FAX_LOG, NULL );
@@ -254,30 +268,29 @@ int main _P2( (argc, argv),
     /* parse switches (-> conf_sf.c) and read global config file */
     if ( sendfax_parse_args( argc, argv ) == ERROR )
     {
-	exit_usage(argv[0]);
+	exit_usage( argv[0], NULL );
     }
 
     /* read config file (defaults) */
     sendfax_get_config( NULL );
 
+    lprintf( L_MESG, "sendfax: %s", mgetty_version );
+    lprintf( L_NOISE, "%s compiled at %s, %s", __FILE__, __DATE__, __TIME__ );
+
     /* for simplicity, put a few config things into global variables */
     verbose = c_bool( verbose );
 
     argidx = optind;
 
+    /* fax number given? */
     if ( argidx == argc )
     {
-	exit_usage(argv[0]);
+	exit_usage( argv[0], "no fax number specified" );
     }
     fac_tel_no = argv[ argidx++ ];
 
     lprintf( L_MESG, "sending fax to %s", fac_tel_no );
 
-    if ( ! c_bool(fax_poll_wanted) && argidx == argc )
-    {
-	exit_usage(argv[0]);
-    }
-
     /* check, if all the arguments passed are normal files and
      * readable
      */
@@ -286,12 +299,24 @@ int main _P2( (argc, argv),
 	lprintf( L_MESG, "checking %s", argv[i] );
 	if ( access( argv[i], R_OK ) == -1 )
 	{
+	    if ( errno == ENOENT && i == argidx 	/* first file */
+		 && c_bool( rename_files ) )		/* and '-r' set */
+	    {
+		argidx++; continue;    			/* just skip */
+	    }
+	    
 	    lprintf( L_ERROR, "cannot access %s", argv[i] );
 	    fprintf( stderr, "%s: cannot access %s\n", argv[0], argv[i]);
 	    exit(1);
 	}
     }
 
+    /* check if any files specified / left */
+    if ( ! c_bool(fax_poll_wanted) && argidx == argc )
+    {
+	exit_usage( argv[0], "no files to send" );
+    }
+
     /* if modem on stdin, shut off blurb */
     if ( c_bool(use_stdin) ) verbose = FALSE;
     
@@ -432,8 +457,9 @@ int main _P2( (argc, argv),
     sprintf( buf, "%s%s", c_string(dial_prefix), fac_tel_no );
     if ( fax_command( buf, "OK", fd ) == ERROR )
     {
-	lprintf( L_AUDIT, "failed dialing, phone=\"%s\", +FHS:%d, time=%ds",
-		 fac_tel_no, fax_hangup_code, ( time(NULL)-call_start ) );
+	lprintf( L_AUDIT, "failed dialing, phone=\"%s\", +FHS:%02d, dev=%s, time=%ds, acct=\"%s\"",
+		 fac_tel_no, fax_hangup_code, Device,
+		 ( time(NULL)-call_start ), c_string(acct_handle) );
 
 	/* close fax line */
 	fax_close( fd );
@@ -449,6 +475,12 @@ int main _P2( (argc, argv),
 	        printf( "dial failed (BUSY/NO ANSWER)\n" );
 	    exit(4);
 	}
+	else if ( fax_hangup_code == FHUP_NODIAL )
+	{
+	    if ( verbose )
+	        printf( "dial failed (NO DIALTONE)\n" );
+	    exit(5);
+	}
 	else
 	{
 	    fprintf( stderr, "\n%s: dial %s failed (ERROR / NO CARRIER)\n",
@@ -468,8 +500,13 @@ int main _P2( (argc, argv),
 #endif	/* sun */
     tio_set( fd, &fax_tio );
 
+    lprintf( L_NOISE, "sendfax: honouring DCD (carrier) drops now" );
+#else
+    lprintf( L_NOISE, "sendfax: FAX_SEND_IGNORE_CARRIER active" );
 #endif	/* !FAX_SEND_IGNORE_CARRIER */
 
+    total_pages = argc-argidx;		/* for statistics */
+
     /* process all files to send / abort, if Modem sent +FHNG result */
 
     tries = 0;
@@ -506,7 +543,8 @@ int main _P2( (argc, argv),
 	
 	fax_page_tx_status = -1;	/* set by fax_send_page() */
 
-	if ( fax_send_page( argv[ argidx ], &fax_tio, ppm, fd ) == ERROR )
+	if ( fax_send_page( argv[ argidx ],
+			    &total_bytes, &fax_tio, ppm, fd ) == ERROR )
 	{
 	    break;
 	}
@@ -550,6 +588,7 @@ int main _P2( (argc, argv),
 		{
 		    if ( verbose )
 		       printf( "sending page again (retry %d)\n", tries );
+		    total_resent++;
 		    continue;	/* don't go to next page */
 		}
 	    }
@@ -568,6 +607,20 @@ int main _P2( (argc, argv),
 	}
 
 	if ( fax_hangup && fax_hangup_code != 0 ) break;
+
+	/* page transmitted successfully, rename file to ".done" */
+	if ( c_bool( rename_files ) )
+	{
+	    char done[MAXPATH+6];
+	    if ( strlen( argv[argidx] ) > sizeof(done)-6 )
+	        fprintf( stderr, "file name %s too long\n", argv[argidx] );
+	    else
+	    {
+		sprintf( done, "%s.done", argv[argidx] );
+		if ( rename( argv[argidx], done ) == -1 )
+		    lprintf( L_ERROR, "can't rename work file to %s", done );
+	    }
+	}
 	
 	argidx++;		/* next page */
 	tries=0;		/* no tries yet */
@@ -575,8 +628,9 @@ int main _P2( (argc, argv),
 
     if ( argidx < argc || ( fax_hangup && fax_hangup_code != 0 ) )
     {
-	lprintf( L_AUDIT, "failed transmitting %s: +FHS:%2d, time=%ds",
-		 argv[argidx], fax_hangup_code, ( time(NULL)-call_start ) );
+	lprintf( L_AUDIT, "failed transmitting %s: phone=\"%s\", +FHS:%02d, dev=%s, time=%ds, acct=\"%s\"",
+		 argv[argidx], fac_tel_no, fax_hangup_code, Device,
+		 ( time(NULL)-call_start ), c_string(acct_handle) );
 
 	fprintf( stderr, "\n%s: FAILED to transmit '%s'.\n",
 		         argv[0], argv[argidx] );
@@ -608,6 +662,11 @@ int main _P2( (argc, argv),
 	}
 	else
 	{
+	    /* class 2.0 modems use the correct byte order, Rockwell-
+	     * compatible class 2 modems get it wrong.
+	     */
+	    if ( modem_type == Mt_class2_0 ) fax_set_bor( fd, 1 );
+	    
 	    /* switch to fax receiver flow control */
 	    tio_set_flow_control( fd, &fax_tio,
 				 (FAXREC_FLOW) & (FLOW_HARD|FLOW_XON_IN) );
@@ -616,8 +675,9 @@ int main _P2( (argc, argv),
 			        -1, -1, -1 ) == ERROR )
 	    {
 		fprintf( stderr, "warning: polling failed\n" );
-		lprintf( L_AUDIT, "failed: polling failed, +FHS:%2d, time=%ds",
-			 fax_hangup_code, ( time(NULL)-call_start ) );
+		lprintf( L_AUDIT, "failed: polling failed, phone=\"%s\", +FHS:%02d, dev=%s, time=%ds, acct=\"%s\"",
+			 fac_tel_no, fax_hangup_code, Device,
+			 ( time(NULL)-call_start ), c_string(acct_handle) );
 		fax_close( fd );
 		exit(12);
 	    }
@@ -627,7 +687,9 @@ int main _P2( (argc, argv),
 
     fax_close( fd );
 
-    lprintf( L_AUDIT, "success, phone=\"%s\", time=%ds",
-	              fac_tel_no, ( time(NULL)-call_start ) );
+    lprintf( L_AUDIT, "success, phone=\"%s\", dev=%s, time=%ds, pages=%d(+%d), bytes=%d, acct=\"%s\"",
+	              fac_tel_no, Device, ( time(NULL)-call_start ),
+	              total_pages, total_resent, total_bytes,
+	              c_string(acct_handle) );
     return 0;
 }
