@@ -1,4 +1,4 @@
-#ident "@(#)syslibs.h	3.1 95/08/30 Copyright (c) Gert Doering"
+#ident "@(#)syslibs.h	3.2 96/03/04 Copyright (c) Gert Doering"
 
 /* Include stdlib.h / malloc.h, depending on the O/S
  */
@@ -10,3 +10,7 @@
 #if !defined( __bsdi__ ) && !defined(__FreeBSD__) && !defined(NeXT)
 #include <malloc.h>
 #endif
+
+#ifdef NEXTSGTTY		/* NeXT, not POSIX subsystem */
+# include <libc.h>
+#endif
