@@ -1,4 +1,4 @@
-#ident "@(#)tio.h	3.1 95/08/30 Copyright (c) 1993 Gert Doering"
+#ident "@(#)tio.h	3.10 96/10/10 Copyright (c) 1993 Gert Doering"
 
 #ifndef __TIO_H__
 #define __TIO_H__
@@ -9,9 +9,16 @@
  *
  */
 
+#ifdef NEXTSGTTY
+# define BSD_SGTTY
+# undef POSIX_TERMIOS
+# undef SYSV_TERMIO
+#endif
+
 #if !defined( POSIX_TERMIOS ) && !defined( BSD_SGTTY ) && !defined( SYSV_TERMIO)
 # if defined(linux) || defined(sunos4) || defined(_AIX) || defined(BSD) || \
-     defined(SVR4) || defined(solaris2) || defined(m88k) || defined(M_UNIX)
+     defined(SVR4) || defined(solaris2) || defined(m88k) || defined(M_UNIX) ||\
+     defined(__sgi)
 #  define POSIX_TERMIOS
 # else
 #  define SYSV_TERMIO
@@ -37,6 +44,13 @@ typedef struct termios TIO;
 typedef struct sgttyb TIO;
 #endif
 
+/* on SCO and other SVR3 systems, the TIOCMGET calls are only available
+ * with special drivers, like the digiboard drivers, or my hacked "FAS"
+ */
+#ifdef USE_FAS_TIOCMGET
+# include <sys/fas.h>
+#endif
+
 /* define some types for gettydefs.c */
 
 #ifdef SYSV_TERMIO
@@ -58,6 +72,13 @@ typedef tcflag_t tioflag_t;
 #include "cannot use /etc/gettydefs with sgtty (yet?)"
 #endif
 
+/* SVR4 came up with a new method of setting h/w flow control */
+/* unfortunately, it's broken in 4.2 and Solaris2, and not there in IRIX! */
+#if defined(SVR4) && \
+    !defined(SVR42) && !defined(solaris2) && !defined(sgi)
+# define USE_TERMIOX
+#endif
+
 /* if not defined in the default header files, #define some important things
  */
 #ifdef _AIX
@@ -140,12 +161,32 @@ typedef tcflag_t tioflag_t;
 #define FLOW_XON_OUT	0x04		/* send data, honor xon/xoff */
 #define FLOW_SOFT	(FLOW_XON_IN | FLOW_XON_OUT)
 #define FLOW_BOTH	(FLOW_HARD | FLOW_SOFT )
+#define FLOW_XON_IXANY	0x08		/* set IXANY flag together with IXON */
 
 /* queue selection flags (for tio_flush_queue) */
 #define TIO_Q_IN	0x01		/* incoming data queue */
 #define TIO_Q_OUT	0x02		/* outgoing data queue */
 #define TIO_Q_BOTH	( TIO_Q_IN | TIO_Q_OUT )
 
+/* RS232 line status flags */
+/* system flags are used if available, otherwise we define our own */
+#ifdef TIOCM_DTR
+# define TIO_F_SYSTEM_DEFS
+# define TIO_F_DTR TIOCM_DTR
+# define TIO_F_DSR TIOCM_DSR
+# define TIO_F_RTS TIOCM_RTS
+# define TIO_F_CTS TIOCM_CTS
+# define TIO_F_DCD TIOCM_CAR
+# define TIO_F_RI  TIOCM_RNG
+#else
+# define TIO_F_DTR 0x001
+# define TIO_F_DSR 0x002
+# define TIO_F_RTS 0x004
+# define TIO_F_CTS 0x008
+# define TIO_F_DCD 0x010
+# define TIO_F_RI  0x020
+#endif
+
 /* function prototypes */
 int  tio_get _PROTO (( int fd, TIO *t ));
 int  tio_set _PROTO (( int fd, TIO *t ));
@@ -164,6 +205,10 @@ void tio_carrier     _PROTO (( TIO *t, int carrier_sensitive ));
 int  tio_toggle_dtr  _PROTO(( int fd, int msec_wait ));
 int  tio_flush_queue _PROTO(( int fd, int queue ));
 int  tio_flow        _PROTO(( int fd, int restart_output ));
+int  tio_break       _PROTO(( int fd ));
+int  tio_drain_output _PROTO(( int fd ));
+
+int  tio_get_rs232_lines _PROTO(( int fd ));		/* get line status */
 
 #ifdef USE_GETTYDEFS
 typedef struct {
